from mock import patch, Mock
from django.test import TestCase
from django.test.client import RequestFactory

from chimps.views import SubscribeView


class ViewTestCase(TestCase):
    def test_view_calls_api_correctly(self):
        """
        Test MailSnake api is called on successfull subscription.
        """

        data = {'name': 'Test Name', 'email': 'test@example.com'}
        request = RequestFactory().post('', data=data)
        view = SubscribeView.as_view()
        mc_api = Mock()

        with patch('chimps.views.MailSnake') as mocked_snake:
            mocked_snake.return_value = mc_api
            view(request)

            kwargs = {
                'double_optin': True,
                'merge_vars': {'NAME': 'Test Name'},
                'email_address': 'test@example.com',
                'id': 'test-mailchimp-list-id',
                'update_existing': True
            }
            mc_api.listSubscribe.assert_called_with(**kwargs)



from django.conf.urls import patterns, url
from django.views.generic import TemplateView
from .views import SubscribeView


urlpatterns = patterns('',
    url(r'^subscribe/$', SubscribeView.as_view(),
        name='chimps_subscribe'),
    url(r'^subscribed/$',
        TemplateView.as_view(template_name='chimps/subscribed.html'),
        name='chimps_subscribed'),
)



from django import forms
from django.core.urlresolvers import reverse
from django.conf import settings
from django.views.generic import FormView
from django.utils.translation import ugettext as _
from mailsnake import MailSnake


class SubscriptionForm(forms.Form):
    name = forms.CharField(label=_('Name'), max_length=100)
    email = forms.EmailField(label=_('Email'))


class SubscribeView(FormView):
    template_name = 'chimps/subscribe.html'
    form_class = SubscriptionForm

    def get_success_url(self):
        return reverse('chimps_subscribed')

    def form_valid(self, form):
        ms = MailSnake(getattr(settings, 'MAILCHIMP_KEY'))
        double_optin = getattr(settings, 'MAILCHIMP_CONFIRM', True)
        list_id = getattr(settings, 'MAILCHIMP_LIST_ID', None)

        if not list_id:
            list_id = ms.lists()['data'][0]['id']

        ms.listSubscribe(id=list_id, email_address=form.cleaned_data['email'],
                         merge_vars={'NAME': form.cleaned_data['name']},
                         update_existing=True, double_optin=double_optin)

        return super(SubscribeView, self).form_valid(form)



# Django settings for django-chimps project.

DEBUG = True
TEMPLATE_DEBUG = DEBUG
DEBUG_PROPAGATE_EXCEPTIONS = True

ALLOWED_HOSTS = ['*']

ADMINS = (
    # ('Your Name', 'your_email@domain.com'),
)

MANAGERS = ADMINS

MAILCHIMP_KEY = 'test-mailchimp-api-key'
MAILCHIMP_LIST_ID = 'test-mailchimp-list-id'

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'sqlite.db',
        'USER': '',
        'PASSWORD': '',
        'HOST': '',
        'PORT': '',
    }
}

CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
    }
}

TIME_ZONE = 'Europe/London'
LANGUAGE_CODE = 'en-uk'

SITE_ID = 1

USE_I18N = True
USE_L10N = True

MEDIA_ROOT = ''
MEDIA_URL = ''

STATIC_URL = '/static/'
STATIC_ROOT = ''

SECRET_KEY = 'u@x-aj9(hoh#rb-^ymf#g2jx_hp0vj7u5#b@ag1n^seu9e!%cy'

TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
)

ROOT_URLCONF = 'chimps.runtests.urls'

TEMPLATE_DIRS = (
)

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'chimps',
)

AUTH_USER_MODEL = 'auth.User'



from django import template
from ..views import SubscribeView


register = template.Library()


@register.inclusion_tag('chimps/form.html')
def chimps_form():
    view = SubscribeView()
    return {'form': view.get_form_class()}



from django.core.management.base import NoArgsCommand
from django.conf import settings
from mailsnake import MailSnake


class Command(NoArgsCommand):
    def handle_noargs(self, **kwargs):
        ms = MailSnake(getattr(settings, 'MAILCHIMP_KEY'))
        for lst in ms.lists()['data']:
            print('Name: {name}\nID: {id}\n'.format(**lst))
